import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import { FormBuilder, FormControl } from '@angular/forms';

import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  delay,
  map,
  merge,
  Observable,
  of,
  startWith,
  Subject,
  switchMap,
  take,
  takeUntil,
  timer,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { Partner } from '../models/partner';
import { PartnerPagination } from '../models/partner-pagination';
import { PartnerService } from '../services/partner.service';
import { ActivatedRoute, Router } from '@angular/router';
import {
  RegistComposeComponent,
  RegistComposeResult,
} from '../compose/regist-compose.component';
import { CashComposeComponent } from '../compose/cash-compose.component';
import { CommissionComposeComponent } from '../compose/commission-compose.component';
import { CompComposeComponent } from '../compose/comp-compose.component';
import { CouponComposeComponent } from '../compose/coupon-compose.component';
import { MessageComposeComponent } from '../compose/message-compose.component';
import { SignoutComposeComponent } from '../compose/signout-compose.component';

import { MatDialog } from '@angular/material/dialog';
import { SiteService } from 'app/modules/polyglot/site/services/site.service';
import { Site } from 'app/modules/proto/models/site_pb';
import { MemberService } from 'app/modules/polyglot/member/services/member.service';
import { MemberModel } from 'app/modules/proto/models/member_pb';
import {
  ListMembersRequest,
  ListMembersResponse,
} from 'app/modules/proto/c2se/member_pb';
import { Pagination } from 'app/modules/proto/protobuf/pagination_pb';
import { ApiService } from 'app/modules/polyglot/api/services/api.service';
@Component({
  selector: 'partner-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .partner-grid {
        /*                     관리   요율  매장  아이디  등급 */
        grid-template-columns: 40px 40px 100px auto 40px;

        @screen sm {
          /*                     관리   요율  매장  아이디  등급 예금주 */
          grid-template-columns: 40px 40px 100px auto 40px 140px;
        }

        @screen md {
          /*                     관리   요율  매장  아이디  등급  예금주 정산 보유금 */
          grid-template-columns: 80px auto 80px 40px 40px 120px 120px 120px 120px 120px 100px;
        }

        @screen lg {
          /*                     관리  아이디 등급  예금주 정산  보유금   충전  환전금 수익금 가입   비고 */
          grid-template-columns: 60px auto 60px 40px 40px 100px 100px 100px 100px 120px 100px;
        }
      }
      .mat-tooltip {
        white-space: pre-line;
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  /* partners$!: Observable<Partner[] | undefined>; */

  listPartner$!: Observable<MemberModel[] | undefined>;

  isLoading = false;
  searchInputControl = new FormControl();
  selectedPartner?: Partner;
  pagination?: PartnerPagination;

  __members: MemberModel[] = [];

  title: string = '';
  btnTitle: string = '';
  memberClassId = '';

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _activatedRoute: ActivatedRoute,
    private _partnerService: PartnerService,
    private __memberService: MemberService,
    private __apiService: ApiService,

    private router: Router,
    private _matDialog: MatDialog
  ) {
    switch (this.router.url) {
      case '/member/partner/all':
        this.title = '전체목록';
        this.btnTitle = '대본등록';
        this.memberClassId = 'ae9b874e-5d0e-4c4d-8432-f45f02691ceb';
        break;
      case '/member/partner/main-office':
        this.title = '본사목록';
        this.btnTitle = '대본등록';
        this.memberClassId = 'ae9b874e-5d0e-4c4d-8432-f45f02691ceb';
        break;
      case '/member/partner/branch':
        this.title = '대본목록';
        this.btnTitle = '부본등록';
        this.memberClassId = 'f25a17e9-5c5f-4e9c-bf80-92a9cedf829c';
        break;
      case '/member/partner/division':
        this.title = '부본목록';
        this.btnTitle = '총판등록';
        this.memberClassId = 'cac7b897-2549-4f04-8415-8868f1dcb1da';
        break;
      case '/member/partner/office':
        this.title = '총판목록';
        this.btnTitle = '매장등록';
        this.memberClassId = 'e11cac11-3825-4f4e-9cd5-39367f23f973';
        break;
      case '/member/partner/store':
        this.title = '매장목록';
        this.btnTitle = '회원등록';
        this.memberClassId = '4598f07a-86d1-42a4-b038-25706683a7cd';
        break;
      default:
        console.log(this.router.url);
        break;
    }
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._partnerService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: PartnerPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products

    this._activatedRoute.data.subscribe((data) => {
      let listMemberResult: ListMembersResponse.Result = data['listPartner'];
      this.listPartner$ = of(listMemberResult.getMembersList());

      /* console.log('dddd', listMemberResult.getMembersList()); */

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });

    this.__apiService
      .listBanks()
      .then((l) => {
        console.log('list', l.getVendorsList());
      })
      .catch((e) => {
        console.log('error', e);
      });
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.detectChanges();

      // If the partner changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;

            let req = new ListMembersRequest();
            let paging = new Pagination();
            paging.setPage(this._paginator.pageIndex);
            paging.setPageSize(this._paginator.pageSize);

            req.setPagination(paging);
            let search = new ListMembersRequest.Search();
            search.setMemberClassId('4b014ef5-3bab-4413-aaf9-b0040a70ec77');
            req.setSearch(search);

            return of(this.__memberService.listMembers(req));
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }

  viewSlot(id: string): void {
    let url: string = 'game/slot';
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __onClickRegist(event: MouseEvent, partner: MemberModel): void {
    const dialogRef = this._matDialog.open(RegistComposeComponent, {
      data: {
        title: this.btnTitle,
        parent: partner,
        classId: this.memberClassId,
      },
    });

    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result: RegistComposeResult) => {
        console.log('Compose dialog was closed: ', result);
      });
  }
  __onClickCash(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CashComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCommission(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CommissionComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickComp(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CompComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCoupon(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CouponComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickMessage(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(MessageComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickSignout(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(SignoutComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }

  __getRateTooltop(): string {
    const tempRate = 0;
    const resultTooltip = `
    요율확인 \n 카지노-바카라: ${tempRate}% \n 카지노-룰렛: ${tempRate}% \n 카지노-드레곤타이거: ${tempRate}% \n 카지노-그외:${tempRate}% \n 슬롯:${tempRate}% \n 카지노루징: ${tempRate}% \n 슬롯루징: ${tempRate}%
    `;
    return resultTooltip;
  }

  __getStateString(state: any): string {
    console.log(state.getState());

    return '정상';
  }
}
